// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/auth_service_platform.dart';
import 'package:netease_roomkit_interface/src/types/auth_service_models.dart';

class MethodChannelAuthStatusEventSink extends AuthStatusEventSink {
  static final _statusNameMap = {
    'KickOut': NEAuthEvent.kKickOut,
    'TokenExpired': NEAuthEvent.kTokenExpired,
    'IncorrectToken': NEAuthEvent.kIncorrectToken,
    'Forbidden': NEAuthEvent.kForbidden,
    'AccountTokenError': NEAuthEvent.kAccountTokenError,
    'LoggedIn': NEAuthEvent.kLoggedIn,
    'LoggedOut': NEAuthEvent.kLoggedOut,
  };

  final _streamController = StreamController<NEAuthEvent>.broadcast();

  @override
  void onAuthStatusChanged(AuthStatusEvent event) {
    _statusNameMap[event.status]
        .guard((status) => _streamController.add(status));
  }
}

class MethodChannelAuthService extends AuthServicePlatform {
  final _api = AuthServiceApi();
  final _authStatusEventSink = MethodChannelAuthStatusEventSink();

  MethodChannelAuthService() {
    AuthStatusEventSink.setup(_authStatusEventSink);
  }

  @override
  Future<VoidResult> login(String account, String token) {
    return _api
        .login(account, token)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> logout() {
    return _api
        .logout()
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Stream<NEAuthEvent> get onAuthStatusChanged =>
      _authStatusEventSink._streamController.stream;

  @override
  Future<bool> get isLoggedIn => _api.isLoggedIn();
}
