// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

class MethodChannelChatController extends RoomChatControllerPlatform {
  final _api = RoomChatControllerApi();

  @override
  Future<VoidResult> joinChatroom(String roomUuid,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .joinChatroom(roomUuid, chatroomType.index)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> leaveChatroom(String roomUuid,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .leaveChatroom(roomUuid, chatroomType.index)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<RoomMessage>> sendBroadcastTextMessage(
      String roomUuid, String message,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .sendBroadcastTextMessage(roomUuid, message, chatroomType.index)
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.message));
  }

  @override
  Future<NEResult<RoomMessage>> sendDirectTextMessage(
      String roomUuid, String userUuid, String message,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .sendDirectTextMessage(roomUuid, userUuid, message, chatroomType.index)
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.message));
  }

  @override
  Future<NEResult<RoomMessage>> sendGroupTextMessage(
      String roomUuid, List<String> userUuids, String message,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .sendGroupTextMessage(roomUuid, userUuids, message, chatroomType.index)
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.message));
  }

  @override
  Future<NEResult<List<NEChatroomMember>>> fetchChatRoomMembers(
      String roomUuid, NEChatroomMemberQueryType queryType, int limit,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .fetchChatRoomMembers(
            roomUuid, queryType.index, limit, chatroomType.index)
        .then((value) {
      List<NEChatroomMember> neChatRoomMembers = [];
      for (var element in value.chatRoomMembers) {
        neChatRoomMembers.add(
            NEChatroomMember(nick: element?.nick, avatar: element?.avatar));
      }
      return NEResult(
          code: value.code, msg: value.msg, data: neChatRoomMembers);
    });
  }

  @override
  Future<NEResult<RoomMessage>> sendImageMessage(String roomUuid,
      String messageUuid, String imagePath, List<String>? userUuids,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .sendImageMessage(roomUuid, messageUuid, imagePath, chatroomType.index,
            userUuids ?? [])
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.message));
  }

  @override
  Future<NEResult<RoomMessage>> sendFileMessage(String roomUuid,
      String messageUuid, String filePath, List<String>? userUuids,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .sendFileMessage(roomUuid, messageUuid, filePath, chatroomType.index,
            userUuids ?? [])
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.message));
  }

  @override
  Future<VoidResult> downloadAttachment(
    String roomUuid,
    String messageUuid,
  ) {
    return _api
        .downloadAttachment(roomUuid, messageUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> cancelDownloadAttachment(
    String roomUuid,
    String messageUuid,
  ) {
    return _api
        .cancelDownloadAttachment(roomUuid, messageUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<List<RoomMessage>>> fetchChatroomHistoryMessages(
      String roomUuid, NEChatroomHistoryMessageSearchOption option) {
    final opt = ChatroomHistoryMessageSearchOption(
        limit: option.limit,
        startTime: option.startTime,
        order: option.order.index,
        chatroomType: option.chatroomType.index);
    return _api.fetchChatroomHistoryMessages(roomUuid, opt).then((value) =>
        NEResult(
            code: value.code,
            msg: value.msg,
            data: value.messages.whereType<RoomMessage>().toList()));
  }

  @override
  Future<VoidResult> recallChatroomMessage(
      String roomUuid, String messageUuid, int messageTime,
      {NEChatroomType chatroomType = NEChatroomType.common}) {
    return _api
        .recallChatroomMessage(
            roomUuid, messageUuid, messageTime, chatroomType.index)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }
}
