// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

class MethodChannelLiveController extends RoomLiveControllerPlatform {
  final _api = RoomLiveControllerApi();

  @override
  Future<NEResult<NERoomLiveInfo?>> startLive(
      String roomUuid, NERoomLiveRequest request) {
    var liveLayout = request.liveLayout.liveLayout();
    return _api
        .startLive(
            roomUuid,
            RoomLiveRequest(
                title: request.title,
                liveLayout: liveLayout,
                userUuidList: request.userUuidList,
                password: request.password,
                extensionConfig: request.extensionConfig))
        .then((value) {
      NERoomLiveInfo? _liveInfo;
      if (value.liveInfo != null) {
        var liveLayout =
            NERoomLiveLayoutEx.fromLiveLayout(value.liveInfo!.liveLayout);
        _liveInfo = NERoomLiveInfo(
            title: value.liveInfo!.title,
            userUuidList: value.liveInfo!.userUuidList,
            password: value.liveInfo!.password,
            extensionConfig: value.liveInfo!.extensionConfig,
            liveLayout: liveLayout,
            chatRoomId: value.liveInfo!.chatRoomId,
            state: NERoomLiveState.values[value.liveInfo!.state],
            hlsPullUrl: value.liveInfo!.hlsPullUrl,
            httpPullUrl: value.liveInfo!.httpPullUrl,
            pushUrl: value.liveInfo!.pushUrl,
            rtmpPullUrl: value.liveInfo!.rtmpPullUrl,
            taskId: value.liveInfo!.taskId);
      } else {
        _liveInfo = null;
      }
      return NEResult(code: value.code, msg: value.msg, data: _liveInfo);
    });
  }

  @override
  Future<VoidResult> stopLive(String roomUuid) {
    return _api
        .stopLive(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<NERoomLiveInfo?>> updateLive(
      String roomUuid, NERoomLiveRequest request) {
    var liveLayout = request.liveLayout.liveLayout();
    return _api
        .updateLive(
            roomUuid,
            RoomLiveRequest(
                title: request.title,
                liveLayout: liveLayout,
                password: request.password,
                userUuidList: request.userUuidList,
                extensionConfig: request.extensionConfig))
        .then((value) {
      NERoomLiveInfo? _liveInfo;
      if (value.liveInfo != null) {
        var liveLayout =
            NERoomLiveLayoutEx.fromLiveLayout(value.liveInfo!.liveLayout);
        _liveInfo = NERoomLiveInfo(
            extensionConfig: value.liveInfo!.extensionConfig,
            title: value.liveInfo!.title,
            userUuidList: value.liveInfo!.userUuidList,
            password: value.liveInfo!.password,
            liveLayout: liveLayout,
            state: NERoomLiveState.values[value.liveInfo!.state],
            chatRoomId: value.liveInfo!.chatRoomId,
            hlsPullUrl: value.liveInfo!.hlsPullUrl,
            httpPullUrl: value.liveInfo!.httpPullUrl,
            pushUrl: value.liveInfo!.pushUrl,
            rtmpPullUrl: value.liveInfo!.rtmpPullUrl,
            taskId: value.liveInfo!.taskId);
      } else {
        _liveInfo = null;
      }
      return NEResult(code: value.code, msg: value.msg, data: _liveInfo);
    });
  }

  @override
  Future<NEResult<NERoomLiveInfo?>> getLiveInfo(String roomUuid) {
    return _api.getLiveInfo(roomUuid).then((value) {
      NERoomLiveInfo? _liveInfo;
      if (value.liveInfo != null) {
        var liveLayout =
            NERoomLiveLayoutEx.fromLiveLayout(value.liveInfo!.liveLayout);
        _liveInfo = NERoomLiveInfo(
            title: value.liveInfo!.title,
            userUuidList: value.liveInfo!.userUuidList,
            password: value.liveInfo!.password,
            liveLayout: liveLayout,
            chatRoomId: value.liveInfo!.chatRoomId,
            hlsPullUrl: value.liveInfo!.hlsPullUrl,
            httpPullUrl: value.liveInfo!.httpPullUrl,
            state: NERoomLiveState.values[value.liveInfo!.state],
            pushUrl: value.liveInfo!.pushUrl,
            rtmpPullUrl: value.liveInfo!.rtmpPullUrl,
            extensionConfig: value.liveInfo!.extensionConfig,
            taskId: value.liveInfo!.taskId);
      }
      return NEResult(code: value.code, msg: value.msg, data: _liveInfo);
    });
  }

  @override
  Future<VoidResult> addLiveStreamTask(
      String roomUuid, NERoomLiveStreamTaskInfo taskInfo) {
    return _api
        .addLiveStreamTask(roomUuid, _convertTaskInfo(taskInfo))
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> updateLiveStreamTask(
      String roomUuid, NERoomLiveStreamTaskInfo taskInfo) {
    return _api
        .updateLiveStreamTask(roomUuid, _convertTaskInfo(taskInfo))
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> removeLiveStreamTask(String roomUuid, String taskID) {
    return _api
        .removeLiveStreamTask(roomUuid, taskID)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  RoomLiveStreamTaskInfo _convertTaskInfo(NERoomLiveStreamTaskInfo info) {
    return RoomLiveStreamTaskInfo(
      taskID: info.taskId,
      streamURL: info.streamUrl,
      serverRecordEnabled: info.serverRecordEnabled,
      lsMode: info.mode.index,
      layout: _convertLayout(info.layout),
      config: _convertConfig(info.config ?? NERoomLiveConfig()),
      extraInfo: info.extraInfo,
    );
  }

  RoomLiveConfig _convertConfig(NERoomLiveConfig config) {
    var sampleRate = 48000;
    switch (config.sampleRate) {
      case NERoomLiveStreamAudioSampleRate.kAudioSampleRate32000:
        sampleRate = 32000;
        break;
      case NERoomLiveStreamAudioSampleRate.kAudioSampleRate44100:
        sampleRate = 44100;
        break;
      case NERoomLiveStreamAudioSampleRate.kAudioSampleRate48000:
        sampleRate = 48000;
        break;
    }
    return RoomLiveConfig(
      singleVideoPassthrough: config.singleVideoPassthrough,
      audioBitrate: config.audioBitrate,
      sampleRate: sampleRate,
      channels: config.channels,
      audioCodecProfile: config.audioCodecProfile.index,
    );
  }

  RoomLiveStreamLayout _convertLayout(NERoomLiveStreamLayout layout) {
    return RoomLiveStreamLayout(
      width: layout.width,
      height: layout.height,
      backgroundColor: layout.backgroundColor,
      users: layout.userTranscodingList
          .map((e) => _convertTranscoding(e))
          .toList(),
      bgImage: _convertImage(layout.backgroundImg) ??
          RoomLiveStreamImageInfo(height: 0, width: 0, x: 0, y: 0),
    );
  }

  RoomLiveStreamUserTranscoding? _convertTranscoding(
      NERoomLiveStreamUserTranscoding? transcoding) {
    if (transcoding == null) {
      return null;
    }
    return RoomLiveStreamUserTranscoding(
      userUuid: transcoding.userUuid,
      videoPush: transcoding.videoPush,
      x: transcoding.x,
      y: transcoding.y,
      width: transcoding.width,
      height: transcoding.height,
      audioPush: transcoding.audioPush,
      zOrder: transcoding.zOrder,
      adaption: transcoding.adaption.index,
    );
  }

  RoomLiveStreamImageInfo? _convertImage(NERoomLiveStreamImageInfo? info) {
    if (info == null) {
      return null;
    }
    return RoomLiveStreamImageInfo(
      url: info.url,
      x: info.x,
      y: info.y,
      width: info.width,
      height: info.height,
    );
  }
}
