// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';
import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/message_service_platform.dart';
import '../types/room_service_models.dart';

class MethodChannelMessageEventSink extends MessageEventSink {
  final _listeners = <MessageEventSink>[];

  void addMessageListener(MessageEventSink sink) {
    _listeners.add(sink);
  }

  void removeMessageListener(MessageEventSink sink) {
    _listeners.remove(sink);
  }

  @override
  void onReceivePassThroughMessage(PassThroughMessageEvent messageEvent) {
    for (var element in _listeners) {
      element.onReceivePassThroughMessage(messageEvent);
    }
  }
}

class MethodChannelMessageService extends MessageChannelServicePlatform {
  final _eventDispatcher = MethodChannelMessageEventSink();
  final _api = MessageServiceApi();

  MethodChannelMessageService() {
    MessageEventSink.setup(_eventDispatcher);
  }

  @override
  void addMessageListener(MessageEventSink sink) {
    _eventDispatcher.addMessageListener(sink);
  }

  @override
  void removeMessageListener(MessageEventSink sink) {
    _eventDispatcher.removeMessageListener(sink);
  }

  @override
  Future<VoidResult> sendCustomMessage(
      String? roomUuid, String userUuid, int commandId, String data,
      {NEInjectedAuthorization? injectedAuthorization}) {
    return _api
        .sendCustomMessage(
          roomUuid,
          userUuid,
          commandId,
          data,
          injectedAuthorization != null
              ? InjectedAuthorization(
                  appKey: injectedAuthorization.appKey,
                  user: injectedAuthorization.user,
                  token: injectedAuthorization.token,
                )
              : null,
        )
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }
}
