// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:async';

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/room_context_platform.dart';

class MethodChannelRoomEventSink extends RoomEventSink {
  final _listeners = <String, Set<RoomEventSink>>{};

  void registerEventListener(String roomUuid, RoomEventSink listener) {
    _listeners.putIfAbsent(roomUuid, () => <RoomEventSink>{}).add(listener);
  }

  void unregisterEventListener(String roomUuid, RoomEventSink listener) {
    _listeners[roomUuid]?.remove(listener);
  }

  @override
  void onMemberAudioMuteChanged(MemberToggleEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberAudioMuteChanged(event);
    });
  }

  @override
  void onMemberJoinChatroom(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberJoinChatroom(event);
    });
  }

  @override
  void onMemberJoinRoom(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberJoinRoom(event);
    });
  }

  @override
  void onMemberJoinRtcChannel(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberJoinRtcChannel(event);
    });
  }

  @override
  void onMemberLeaveChatroom(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberLeaveChatroom(event);
    });
  }

  @override
  void onMemberLeaveRoom(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberLeaveRoom(event);
    });
  }

  @override
  void onMemberLeaveRtcChannel(MembersEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberLeaveRtcChannel(event);
    });
  }

  @override
  void onMemberPropertiesChanged(PropertyEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberPropertiesChanged(event);
    });
  }

  @override
  void onMemberPropertiesDeleted(PropertyEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberPropertiesDeleted(event);
    });
  }

  @override
  void onMemberRoleChanged(MemberRoleChangeEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberRoleChanged(event);
    });
  }

  @override
  void onMemberScreenShareStateChanged(MemberToggleEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberScreenShareStateChanged(event);
    });
  }

  @override
  void onMemberVideoMuteChanged(MemberToggleEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberVideoMuteChanged(event);
    });
  }

  @override
  void onReceiveChatroomMessages(RoomMessageEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onReceiveChatroomMessages(event);
    });
  }

  @override
  void onChatroomMessageAttachmentProgress(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onChatroomMessageAttachmentProgress(event);
    });
  }

  @override
  void onRoomPropertiesChanged(PropertyEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRoomPropertiesChanged(event);
    });
  }

  @override
  void onRoomPropertiesDeleted(PropertyEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRoomPropertiesDeleted(event);
    });
  }

  @override
  void onRtcChannelError(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRtcChannelError(event);
    });
  }

  @override
  void onRoomEnd(String roomUuid, String reason) {
    _listeners[roomUuid]?.forEach((element) {
      element.onRoomEnd(roomUuid, reason);
    });
  }

  @override
  void onMemberWhiteboardShareStateChanged(MemberToggleEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberWhiteboardShareStateChanged(event);
    });
  }

  @override
  void onWhiteboardError(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onWhiteboardError(event);
    });
  }

  @override
  void onRoomLockStateChanged(RoomToggleEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRoomLockStateChanged(event);
    });
  }

  @override
  void onRoomLiveStateChanged(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRoomLiveStateChanged(event);
    });
  }

  @override
  void onMemberNameChanged(MemberNameChangeEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberNameChanged(event);
    });
  }

  @override
  void onRtcAudioVolumeIndication(RtcVolumeEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRtcAudioVolumeIndication(event);
    });
  }

  @override
  void onRtcAudioOutputDeviceChanged(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRtcAudioOutputDeviceChanged(event);
    });
  }

  @override
  void onLocalAudioVolumeIndicationwithVad(LocalVolumeEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onLocalAudioVolumeIndicationwithVad(event);
    });
  }

  @override
  void onRtcVirtualBackgroundSourceEnabled(
      RtcVirtualBackgroundSourceEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onRtcVirtualBackgroundSourceEnabled(event);
    });
  }

  @override
  Future<String?> onWhiteboardShowFileChooser(
      WhiteboardShowFileChooserTypeEvent event) {
    var result;
    _listeners[event.roomUuid]?.forEach((element) {
      result = element.onWhiteboardShowFileChooser(event);
    });
    return result;
  }

  @override
  void onAudioMixingStateChanged(CommonEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onAudioMixingStateChanged(event);
    });
  }

  @override
  void onRoomRemainingSecondsRenewed(String roomUuid, int remainingSeconds) {
    _listeners[roomUuid]?.forEach((element) {
      element.onRoomRemainingSecondsRenewed(roomUuid, remainingSeconds);
    });
  }

  @override
  void onRoomConnectStateChanged(String roomUuid, int state) {
    _listeners[roomUuid]?.forEach((element) {
      element.onRoomConnectStateChanged(roomUuid, state);
    });
  }

  @override
  void onRoomCloudRecordStateChanged(
      String roomUuid, int state, String? operateBy) {
    _listeners[roomUuid]?.forEach((element) {
      element.onRoomCloudRecordStateChanged(roomUuid, state, operateBy);
    });
  }

  @override
  void onMemberAudioConnectStateChanged(
      MemberAudioConnectStateChangedEvent event) {
    _listeners[event.roomUuid]?.forEach((element) {
      element.onMemberAudioConnectStateChanged(event);
    });
  }
}

class MethodChannelRoomContext extends RoomContextPlatform {
  final _api = RoomContextApi();
  final _eventDispatcher = MethodChannelRoomEventSink();

  MethodChannelRoomContext() {
    RoomEventSink.setup(_eventDispatcher);
  }

  @override
  void registerEventListener(String roomUuid, RoomEventSink listener) {
    _eventDispatcher.registerEventListener(roomUuid, listener);
  }

  @override
  void unregisterEventListener(String roomUuid, RoomEventSink listener) {
    _eventDispatcher.unregisterEventListener(roomUuid, listener);
  }

  @override
  Future<VoidResult> changeMyName(String roomUuid, String name) {
    return _api
        .changeMyName(roomUuid, name)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> changeMemberName(
      String roomUuid, String userUuid, String name) {
    return _api
        .changeMemberName(roomUuid, userUuid, name)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> changeMemberRole(
      String roomUuid, String userUuid, String role) {
    return _api
        .changeMemberRole(roomUuid, userUuid, role)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> handOverMyRole(String roomUuid, String userUuid) {
    return _api
        .handOverMyRole(roomUuid, userUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> endRoom(String roomUuid, bool isForce) {
    return _api
        .endRoom(roomUuid, isForce)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> kickMemberOut(String roomUuid, String userUuid) {
    return _api
        .kickMemberOut(roomUuid, userUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> leaveRoom(String roomUuid) {
    return _api
        .leaveRoom(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> updateRoomProperty(
      String roomUuid, String key, String value,
      {String? associatedUuid}) {
    return _api
        .updateRoomProperty(roomUuid, key, value, associatedUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> deleteRoomProperty(String roomUuid, String key) {
    return _api
        .deleteRoomProperty(roomUuid, key)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> updateMemberProperty(
      String roomUuid, String userUuid, String key, String value) {
    return _api
        .updateMemberProperty(roomUuid, userUuid, key, value)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> deleteMemberProperty(
      String roomUuid, String userUuid, String key) {
    return _api
        .deleteMemberProperty(roomUuid, userUuid, key)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> lockRoom(String roomUuid) {
    return _api
        .lockRoom(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> unlockRoom(String roomUuid) {
    return _api
        .unlockRoom(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startCloudRecord(String roomUuid) {
    return _api
        .startCloudRecord(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopCloudRecord(String roomUuid) {
    return _api
        .stopCloudRecord(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  Future<NEResult<RoomSnapshot>> rejoinAfterAdmittedToRoom(String roomUuid) {
    return _api.rejoinAfterAdmittedToRoom(roomUuid).then((value) =>
        NEResult(code: value.code, msg: value.msg, data: value.snapshot));
  }
}
