// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/room_rtc_controller_platform.dart';
import 'package:netease_roomkit_interface/src/types/rtc_models.dart';

class MethodChannelRoomRtcController extends RoomRtcControllerPlatform {
  final _api = RoomRtcControllerApi();

  @override
  Future<VoidResult> startPreview(String roomUuid) {
    return _api
        .startPreview(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopPreview(String roomUuid, bool releaseRtc) {
    return _api
        .stopPreview(roomUuid, releaseRtc)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> joinRtcChannel(String roomUuid) {
    return _api
        .joinRtcChannel(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> leaveRtcChannel(String roomUuid) {
    return _api
        .leaveRtcChannel(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> muteMyAudio(String roomUuid, bool enableMediaPub) {
    return _api
        .muteMyAudio(roomUuid, enableMediaPub)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> muteMyVideo(String roomUuid) {
    return _api
        .muteMyVideo(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopScreenShare(String roomUuid) {
    return _api
        .stopScreenShare(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> subscribeRemoteSubStreamVideo(
      String roomUuid, String userUuid, bool subscribe) {
    return _api
        .subscribeRemoteSubStreamVideo(roomUuid, userUuid, subscribe)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> subscribeRemoteVideoStream(
    String roomUuid,
    String userUuid,
    NEVideoStreamType streamType,
    bool subscribe,
  ) {
    return _api
        .subscribeRemoteVideoStream(
            roomUuid, userUuid, streamType.index, subscribe)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> adjustUserPlaybackSignalVolume(
      String roomUuid, String userUuid, int volume) {
    return _api
        .adjustUserPlaybackSignalVolume(roomUuid, userUuid, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> unmuteMyAudio(String roomUuid, bool enableMediaPub) {
    return _api
        .unmuteMyAudio(roomUuid, enableMediaPub)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> unmuteMyVideo(String roomUuid) {
    return _api
        .unmuteMyVideo(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopMemberScreenShare(String roomUuid, String uuid) {
    return _api
        .stopMemberScreenShare(roomUuid, uuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> muteMemberAudio(String roomUuid, String uuid) {
    return _api
        .muteMemberAudio(roomUuid, uuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> muteMemberVideo(String roomUuid, String uuid) {
    return _api
        .muteMemberVideo(roomUuid, uuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> unmuteMemberAudio(String roomUuid, String uuid) {
    return _api
        .unmuteMemberAudio(roomUuid, uuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> unmuteMemberVideo(String roomUuid, String uuid) {
    return _api
        .unmuteMemberVideo(roomUuid, uuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startScreenShare(String roomUuid, {String? iosAppGroup}) {
    return _api
        .startScreenShare(StartScreenShareRequest(
            roomUuid: roomUuid, iosAppGroup: iosAppGroup))
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableAudioVolumeIndication(
      String roomUuid, bool enable, int interval,
      [bool vad = false]) {
    return _api
        .enableAudioVolumeIndication(roomUuid, enable, interval, vad)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<bool> isSpeakerphoneOn(String roomUuid) {
    return _api.isSpeakerphoneOn(roomUuid).then((value) => value.code == 0);
  }

  @override
  Future<VoidResult> setSpeakerphoneOn(String roomUuid, bool on) {
    return _api
        .setSpeakerphoneOn(roomUuid, on)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startAudioDump(String roomUuid, NEAudioDumpType type) {
    return _api
        .startAudioDump(roomUuid, type.index)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopAudioDump(String roomUuid) {
    return _api
        .stopAudioDump(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startRtcChannelMediaRelay(String roomUuid) {
    return _api
        .startRtcChannelMediaRelay(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopRtcChannelMediaRelay(String roomUuid) {
    return _api
        .stopRtcChannelMediaRelay(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startAudioMixing(
      String roomUuid, NECreateAudioMixingOption option) {
    return _api
        .startAudioMixing(
            roomUuid,
            CreateAudioMixingOption(
                path: option.path,
                loopCount: option.loopCount,
                playbackVolume: option.playbackVolume,
                playbackEnabled: option.playbackEnabled,
                sendEnabled: option.sendEnabled,
                sendVolume: option.sendVolume,
                startTimeStamp: option.startTimeStamp,
                sendWithAudioType: option.sendWithAudioType ==
                        NERoomRtcAudioStreamType.kNERtcAudioStreamTypeMain
                    ? AudioStreamType.AudioStreamTypeMain
                    : AudioStreamType.AudioStreamTypeSub))
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> playEffect(
      String roomUuid, int id, NECreateAudioEffectOption option) {
    return _api
        .playEffect(
            roomUuid,
            id,
            CreateAudioEffectOption(
                path: option.path,
                loopCount: option.loopCount,
                playbackVolume: option.playbackVolume,
                playbackEnabled: option.playbackEnabled,
                sendEnabled: option.sendEnabled,
                sendVolume: option.sendVolume,
                startTimeStamp: option.startTimeStamp,
                progressInterval: option.progressInterval,
                sendWithAudioType: option.sendWithAudioType ==
                        NERoomRtcAudioStreamType.kNERtcAudioStreamTypeMain
                    ? AudioStreamType.AudioStreamTypeMain
                    : AudioStreamType.AudioStreamTypeSub))
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopAudioMixing(String roomUuid) {
    return _api
        .stopAudioMixing(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopEffect(String roomUuid, int id) {
    return _api
        .stopEffect(roomUuid, id)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setAudioMixingSendVolume(String roomUuid, int volume) {
    return _api
        .setAudioMixingSendVolume(roomUuid, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setAudioMixingPlaybackVolume(String roomUuid, int volume) {
    return _api
        .setAudioMixingPlaybackVolume(roomUuid, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setEffectSendVolume(String roomUuid, int id, int volume) {
    return _api
        .setEffectSendVolume(roomUuid, id, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setEffectPlaybackVolume(
      String roomUuid, int id, int volume) {
    return _api
        .setEffectPlaybackVolume(roomUuid, id, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopAllEffects(String roomUuid) {
    return _api
        .stopAllEffects(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableEarBack(String roomUuid, bool enable, int volume) {
    return _api
        .enableEarBack(roomUuid, enable, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> switchCamera(String roomUuid) {
    return _api
        .switchCamera(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> addBeautyFilter(String roomUuid, String path) {
    return _api
        .addBeautyFilter(roomUuid, path)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> addBeautySticker(String roomUuid, String path) {
    return _api
        .addBeautySticker(roomUuid, path)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableBeauty(String roomUuid, bool isOpenBeauty) {
    return _api
        .enableBeauty(roomUuid, isOpenBeauty)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> removeBeautyFilter(
    String roomUuid,
  ) {
    return _api
        .removeBeautyFilter(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> removeBeautySticker(
    String roomUuid,
  ) {
    return _api
        .removeBeautySticker(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setBeautyEffect(
      String roomUuid, NERoomBeautyEffectType beautyType, double level) {
    return _api
        .setBeautyEffect(roomUuid,
            NERoomBeautyEffectTypeExtension.fromType(beautyType), level)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setBeautyFilterLevel(String roomUuid, double level) {
    return _api
        .setBeautyFilterLevel(roomUuid, level)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> startBeauty(
    String roomUuid,
  ) {
    return _api
        .startBeauty(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> stopBeauty(
    String roomUuid,
  ) {
    return _api
        .stopBeauty(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableVirtualBackground(String roomUuid, bool enabled,
      NERoomVirtualBackgroundSource backgroundSource) {
    return _api
        .enableVirtualBackground(roomUuid, enabled, backgroundSource)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> adjustRecordingSignalVolume(String roomUuid, int volume) {
    return _api
        .adjustRecordingSignalVolume(roomUuid, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> resumeAudioMixing(String roomUuid) {
    return _api
        .resumeAudioMixing(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setChannelProfile(String roomUuid, int value) {
    return _api
        .setChannelProfile(roomUuid, value)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setClientRole(String roomUuid, int role) {
    return _api
        .setClientRole(roomUuid, role)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setLocalAudioProfile(
      String roomUuid, int profile, int scenario) {
    return _api
        .setLocalAudioProfile(roomUuid, profile, scenario)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setRecordDeviceMute(String roomUuid, bool mute) {
    return _api
        .setRecordDeviceMute(roomUuid, mute)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> pauseAudioMixing(String roomUuid) {
    return _api
        .pauseAudioMixing(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableMediaPub(
      String roomUuid, int mediaType, bool enable) {
    return _api
        .enableMediaPub(roomUuid, mediaType, enable)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> pauseLocalAudioRecording(String roomUuid) {
    return _api
        .pauseLocalAudioRecording(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> resumeLocalAudioRecording(String roomUuid) {
    return _api
        .resumeLocalAudioRecording(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> pauseLocalVideoCapture(String roomUuid) {
    return _api
        .pauseLocalVideoCapture(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> resumeLocalVideoCapture(String roomUuid) {
    return _api
        .resumeLocalVideoCapture(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> adjustPlaybackSignalVolume(String roomUuid, int volume) {
    return _api
        .adjustPlaybackSignalVolume(roomUuid, volume)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableAudioAINS(String roomUuid, bool enable) {
    return _api
        .enableAudioAINS(roomUuid, enable)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> enableLoopbackRecording(String roomUuid, bool enable) {
    return _api
        .enableLoopbackRecording(roomUuid, enable)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<int>> disableEncryption(String roomUuid) {
    return _api.disableEncryption(roomUuid).then((value) =>
        NEResult(code: value.code, msg: value.msg, data: value.data1));
  }

  @override
  Future<NEResult<int>> enableEncryption(
      String roomUuid, String encryptionKey, NEEncryptionMode encryptionMode) {
    return _api
        .enableEncryption(roomUuid, encryptionKey, encryptionMode.index)
        .then((value) =>
            NEResult(code: value.code, msg: value.msg, data: value.data1));
  }

  @override
  Future<VoidResult> subscribeRemoteAudioStream(
      String roomUuid, String userUuid, bool subscribe) {
    return _api
        .subscribeRemoteAudioStream(roomUuid, userUuid, subscribe)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> setParameters(
      String roomUuid, Map<String, Object> params) {
    return _api
        .setParameters(roomUuid, params)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
    ;
  }

  @override
  Future<VoidResult> disconnectMyAudio(String roomUuid) {
    return _api
        .disconnectMyAudio(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<VoidResult> reconnectMyAudio(String roomUuid) {
    return _api
        .reconnectMyAudio(roomUuid)
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }
}
