// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

// import 'package:netease_common/netease_common.dart';
// import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

import 'package:netease_roomkit_interface/src/platform_interface/room_rtc_stats_platform.dart';

class MethodChannelRoomStatsSink extends RoomStatsSink {
  final _listeners = <String, Set<RoomStatsSink>>{};

  @override
  void onRtcStats(String roomUuid, RoomStatsEvent event) {
    _listeners[roomUuid]?.forEach((element) {
      element.onRtcStats(roomUuid, event);
    });
  }

  @override
  void onNetworkQuality(String roomUuid, List<RoomNetworkQuality?> qualities) {
    _listeners[roomUuid]?.forEach((element) {
      element.onNetworkQuality(roomUuid, qualities);
    });
  }

  void registerStatsListener(String roomUuid, RoomStatsSink listener) {
    _listeners.putIfAbsent(roomUuid, () => <RoomStatsSink>{}).add(listener);
  }

  void unregisterStatsListener(String roomUuid, RoomStatsSink listener) {
    _listeners[roomUuid]?.remove(listener);
  }
}

class MethodChannelRoomStats extends RoomStatsPlatform {
  final _eventDispatcher = MethodChannelRoomStatsSink();

  MethodChannelRoomStats() {
    RoomStatsSink.setup(_eventDispatcher);
  }

  void registerStatsListener(String roomUuid, RoomStatsSink listener) {
    _eventDispatcher.registerStatsListener(roomUuid, listener);
  }

  void unregisterStatsListener(String roomUuid, RoomStatsSink listener) {
    _eventDispatcher.unregisterStatsListener(roomUuid, listener);
  }
}
