// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/room_service_platform.dart';
import 'package:netease_roomkit_interface/src/types/chatroom_models.dart';
import 'package:netease_roomkit_interface/src/types/room_service_models.dart';

class MethodChannelRoomService extends RoomServicePlatform {
  final _api = RoomServiceApi();

  @override
  Future<VoidResult> createRoom(params, options) {
    SeatInitParams? seatInitParams;
    if (params.seatInitParams != null) {
      seatInitParams = SeatInitParams(
          seatCount: params.seatInitParams!.seatCount,
          seatRequestApprovalMode:
              params.seatInitParams!.seatRequestApprovalMode,
          seatInvitationConfirmMode:
              params.seatInitParams!.seatInvitationConfirmMode);
    }
    return _api
        .createRoom(
          CreateRoomParams(
            roomUuid: params.roomUuid,
            roomName: params.roomName,
            password: params.password,
            templateId: params.templateId,
            seatInitParams: seatInitParams,
            initialProperties: params.initialRoomProperties,
          ),
          CreateRoomOptions(
            enableRecord: options.enableRecord,
            enableLive: options.enableLive,
            enableRtc: options.enableRtc,
            enableChatroom: options.enableChatroom,
            enableWhiteboard: options.enableWhiteboard,
            enableWaitingRoom: options.enableWaitingRoom,
            // enableSip: options.enableSip,
          ),
        )
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<RoomSnapshot>> joinRoom(
      NEJoinRoomParams params, NEJoinRoomOptions options) {
    return _api
        .joinRoom(
      JoinRoomParams(
        roomUuid: params.roomUuid,
        userName: params.userName,
        avatar: params.avatar,
        password: params.password,
        role: params.role,
        agentMode: params.agentMode,
        injectedAuthorization: params.injectedAuthorization != null
            ? InjectedAuthorization(
                appKey: params.injectedAuthorization!.appKey,
                user: params.injectedAuthorization!.user,
                token: params.injectedAuthorization!.token,
              )
            : null,
        initialProperties: params.initialMyProperties,
      ),
      JoinRoomOptions(
        enableMyAudioDeviceOnJoinRtc: options.enableMyAudioDeviceOnJoinRtc,
        autoSubscribeAudio: options.autoSubscribeAudio,
      ),
    )
        .then((value) {
      return NEResult(code: value.code, msg: value.msg, data: value.snapshot);
    });
  }

  @override
  Future<VoidResult> previewRoom(
      NEPreviewRoomParams params, NEPreviewRoomOptions options) {
    return _api
        .previewRoom(PreviewRoomParams(), PreviewRoomOptions())
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<NEResult<List<RoomRecord?>?>> getRoomCloudRecordList(
      String roomArchiveId) {
    return _api.getRoomCloudRecordList(roomArchiveId).then((value) =>
        NEResult(code: value.code, msg: value.msg, data: value.recordList));
  }

  @override
  Future<NEResult<List<RoomMessage>>> fetchChatroomHistoryMessages(
      String roomArchiveId, NEChatroomHistoryMessageSearchOption option) {
    final opt = ChatroomHistoryMessageSearchOption(
        limit: option.limit,
        startTime: option.startTime,
        order: option.order.index,
        chatroomType: option.chatroomType.index);
    return _api.fetchChatroomHistoryMessages(roomArchiveId, opt).then((value) =>
        NEResult(
            code: value.code,
            msg: value.msg,
            data: value.messages.whereType<RoomMessage>().toList()));
  }

  @override
  Future<NEResult<VoidResult>> downloadAttachment(String messageUuid) {
    return _api.downloadAttachment(messageUuid).then((value) => NEResult(
          code: value.code,
          msg: value.msg,
        ));
  }
}
