// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.
import 'dart:convert';

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:netease_roomkit_interface/src/platform_interface/roomkit_platform.dart';
import 'package:netease_roomkit_interface/src/types/roomkit_options.dart';

class MethodChannelRoomKit extends RoomKitPlatform {
  final _api = RoomKitApi();

  @override
  Future<VoidResult> initialize(NERoomKitOptions options) async {
    return _api
        .initialize(
          InitializeRequest(
            appKey: options.appKey,
            useAssetServerConfig: options.useAssetServerConfig,
            serverUrl: options.serverUrl,
            serverConfigJson: options.serverConfig != null
                ? jsonEncode(options.serverConfig?.toJson())
                : null,
            extras: {
              ...options.extras,
              'framework': 'Flutter',
            },
          ),
        )
        .then((value) => VoidResult(code: value.code, msg: value.msg));
  }

  @override
  Future<int> switchLanguage(String language) {
    return _api.switchLanguage(language);
  }

  @override
  Future<SDKVersions> get sdkVersions {
    return _api.sdkVersions();
  }

  @override
  Future<String> get deviceId {
    return _api.getDeviceId();
  }

  @override
  Future<NEResult<String?>> uploadLog() async {
    return NEResult.successWith(await _api.uploadLog());
  }

  @override
  Future<dynamic> invokeMethod(String method,
      [Map<String, dynamic>? arguments]) {
    return _api.invokeMethod(method, arguments);
  }
}
