// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';

import '../method_channel/method_channel_preview_room_rtc_controller.dart';

abstract class PreviewRoomRtcControllerPlatform extends PlatformInterface {
  static final Object _token = Object();

  PreviewRoomRtcControllerPlatform() : super(token: _token);

  static PreviewRoomRtcControllerPlatform _instance =
      MethodChannelPreviewRoomRtcController();

  static PreviewRoomRtcControllerPlatform get instance => _instance;

  static set instance(PreviewRoomRtcControllerPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  void registerEventListener(PreviewRoomEventSink listener);

  void unregisterEventListener(PreviewRoomEventSink listener);

  Future<VoidResult> startPreview();

  Future<VoidResult> stopPreview(bool releaseRtc);

  Future<VoidResult> switchCamera();

  Future<VoidResult> addBeautyFilter(String path);

  Future<VoidResult> addBeautySticker(String path);

  Future<VoidResult> enableBeauty(bool isOpenBeauty);

  Future<VoidResult> removeBeautyFilter();

  Future<VoidResult> removeBeautySticker();

  Future<VoidResult> startBeauty();

  Future<VoidResult> stopBeauty();

  Future<VoidResult> setBeautyFilterLevel(double level);

  Future<VoidResult> setBeautyEffect(
      NERoomBeautyEffectType beautyType, double level);

  Future<VoidResult> enableVirtualBackground(
      bool enabled, NERoomVirtualBackgroundSource backgroundSource);
}
