// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_roomkit_interface/src/method_channel/method_channel_chat_controller.dart';
import 'package:netease_common/netease_common.dart';

abstract class RoomChatControllerPlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomChatControllerPlatform() : super(token: _token);

  static RoomChatControllerPlatform _instance = MethodChannelChatController();

  static RoomChatControllerPlatform get instance => _instance;

  static set instance(RoomChatControllerPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  Future<VoidResult> joinChatroom(String roomUuid,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<VoidResult> leaveChatroom(String roomUuid,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<RoomMessage>> sendBroadcastTextMessage(
      String roomUuid, String message,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<RoomMessage>> sendDirectTextMessage(
      String roomUuid, String userUuid, String message,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<RoomMessage>> sendGroupTextMessage(
      String roomUuid, List<String> userUuids, String message,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<List<NEChatroomMember>>> fetchChatRoomMembers(
      String roomUuid, NEChatroomMemberQueryType queryType, int limit,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<List<RoomMessage>>> fetchChatroomHistoryMessages(
      String roomUuid, NEChatroomHistoryMessageSearchOption option);

  Future<VoidResult> recallChatroomMessage(
      String roomUuid, String messageUuid, int messageTime,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<RoomMessage>> sendImageMessage(String roomUuid,
      String messageUuid, String imagePath, List<String>? userUuids,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<NEResult<RoomMessage>> sendFileMessage(String roomUuid,
      String messageUuid, String filePath, List<String>? userUuids,
      {NEChatroomType chatroomType = NEChatroomType.common});

  Future<VoidResult> downloadAttachment(
    String roomUuid,
    String messageUuid,
  );

  Future<VoidResult> cancelDownloadAttachment(
    String roomUuid,
    String messageUuid,
  );
}
