// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/method_channel/method_channel_room_context.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

abstract class RoomContextPlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomContextPlatform() : super(token: _token);

  static RoomContextPlatform _instance = MethodChannelRoomContext();

  static RoomContextPlatform get instance => _instance;

  static set instance(RoomContextPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  Future<VoidResult> changeMyName(String roomUuid, String name);

  Future<VoidResult> changeMemberName(
      String roomUuid, String userUuid, String name);

  Future<VoidResult> changeMemberRole(
      String roomUuid, String userUuid, String role);

  Future<VoidResult> handOverMyRole(String roomUuid, String userUuid);

  Future<VoidResult> kickMemberOut(String roomUuid, String userUuid);

  Future<VoidResult> leaveRoom(String roomUuid);

  Future<VoidResult> endRoom(String roomUuid, bool isForce);

  Future<VoidResult> lockRoom(String roomUuid);

  Future<VoidResult> unlockRoom(String roomUuid);

  Future<VoidResult> updateRoomProperty(
      String roomUuid, String key, String value,
      {String? associatedUuid});

  Future<VoidResult> deleteRoomProperty(String roomUuid, String key);

  Future<VoidResult> updateMemberProperty(
      String roomUuid, String userUuid, String key, String value);

  Future<VoidResult> deleteMemberProperty(
      String roomUuid, String userUuid, String key);

  Future<VoidResult> startCloudRecord(String roomUuid);

  Future<VoidResult> stopCloudRecord(String roomUuid);

  Future<NEResult<RoomSnapshot>> rejoinAfterAdmittedToRoom(String roomUuid);

  void registerEventListener(String roomUuid, RoomEventSink listener);

  void unregisterEventListener(String roomUuid, RoomEventSink listener);
}
