// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/method_channel/method_channel_live_controller.dart';

import '../types/room_live_models.dart';

abstract class RoomLiveControllerPlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomLiveControllerPlatform() : super(token: _token);

  static RoomLiveControllerPlatform _instance = MethodChannelLiveController();

  static RoomLiveControllerPlatform get instance => _instance;

  static set instance(RoomLiveControllerPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  Future<NEResult<NERoomLiveInfo?>> startLive(
      String roomUuid, NERoomLiveRequest request);

  Future<VoidResult> stopLive(String roomUuid);

  Future<NEResult<NERoomLiveInfo?>> updateLive(
      String roomUuid, NERoomLiveRequest request);

  Future<NEResult<NERoomLiveInfo?>> getLiveInfo(String roomUuid);

  Future<VoidResult> addLiveStreamTask(
      String roomUuid, NERoomLiveStreamTaskInfo taskInfo);

  Future<VoidResult> updateLiveStreamTask(
      String roomUuid, NERoomLiveStreamTaskInfo taskInfo);

  Future<VoidResult> removeLiveStreamTask(String roomUuid, String taskID);
}
