// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_roomkit_interface/src/method_channel/method_channel_seat_controller.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:netease_common/netease_common.dart';

abstract class RoomSeatControllerPlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomSeatControllerPlatform() : super(token: _token);

  static RoomSeatControllerPlatform _instance = MethodChannelSeatController();

  static RoomSeatControllerPlatform get instance => _instance;

  static set instance(RoomSeatControllerPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  Future<VoidResult> submitSeatRequest(
      String roomUuid, int seatIndex, bool exclusive);

  Future<VoidResult> cancelSeatRequest(String roomUuid);

  Future<VoidResult> leaveSeat(String roomUuid);

  Future<VoidResult> approveSeatRequest(String roomUuid, String user);

  Future<VoidResult> rejectSeatRequest(String roomUuid, String user);

  Future<VoidResult> kickSeat(String roomUuid, String user);

  Future<VoidResult> sendSeatInvitation(
      String roomUuid, int seatIndex, String user);

  Future<VoidResult> cancelSeatInvitation(String roomUuid, String user);

  Future<VoidResult> acceptSeatInvitation(String roomUuid);

  Future<VoidResult> rejectSeatInvitation(String roomUuid);

  Future<VoidResult> closeSeats(String roomUuid, List<int> seatIndices);

  Future<VoidResult> openSeats(String roomUuid, List<int> seatIndices);

  Future<NEResult<NESeatInfo>> getSeatInfo(String roomUuid);

  Future<NEResult<List<NESeatRequestItem>>> getSeatRequestList(String roomUuid);

  Future<NEResult<List<NESeatInvitationItem>>> getSeatInvitationList(
      String roomUuid);

  void registerEventListener(String roomUuid, SeatEventSink listener);

  void unregisterEventListener(String roomUuid, SeatEventSink listener);
}
