// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/method_channel/method_channel_room_service.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';

abstract class RoomServicePlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomServicePlatform() : super(token: _token);

  static RoomServicePlatform _instance = MethodChannelRoomService();

  static RoomServicePlatform get instance => _instance;

  static set instance(RoomServicePlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  Future<VoidResult> createRoom(
      NECreateRoomParams params, NECreateRoomOptions options);

  Future<NEResult<RoomSnapshot>> joinRoom(
      NEJoinRoomParams params, NEJoinRoomOptions options);

  Future<VoidResult> previewRoom(
      NEPreviewRoomParams params, NEPreviewRoomOptions options);

  Future<NEResult<List<RoomRecord?>?>> getRoomCloudRecordList(
      String roomArchiveId);

  Future<NEResult<List<RoomMessage>>> fetchChatroomHistoryMessages(
      String roomArchiveId, NEChatroomHistoryMessageSearchOption option);

  Future<NEResult<VoidResult>> downloadAttachment(String messageUuid);
}
