// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:netease_common/netease_common.dart';
import 'package:netease_roomkit_interface/src/method_channel/method_channel_waiting_room_controller.dart';
import 'package:netease_roomkit_interface/src/pigeon.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import '../types/waiting_room.dart';

abstract class RoomWaitingRoomControllerPlatform extends PlatformInterface {
  static final Object _token = Object();

  RoomWaitingRoomControllerPlatform() : super(token: _token);

  static RoomWaitingRoomControllerPlatform _instance =
      MethodChannelWaitingRoomController();

  static RoomWaitingRoomControllerPlatform get instance => _instance;

  static set instance(RoomWaitingRoomControllerPlatform instance) {
    PlatformInterface.verify(instance, _token);
    _instance = instance;
  }

  void registerEventListener(String roomUuid, WaitingRoomEventSink listener);

  void unregisterEventListener(String roomUuid, WaitingRoomEventSink listener);

  Future<VoidResult> admitMember(String roomUuid, String userUuid);

  Future<VoidResult> expelMember(
    String roomUuid,
    String userUuid,
    bool disallowRejoin,
  );

  Future<VoidResult> putInWaitingRoom(String roomUuid, String userUuid);

  Future<VoidResult> enableWaitingRoomOnEntry(String roomUuid);

  Future<VoidResult> disableWaitingRoomOnEntry(String roomUuid, bool admitAll);

  Future<NEResult<List<NEWaitingRoomMember>>> getMemberList(
      String roomUuid, int joinTime, int size, bool asc);

  Future<VoidResult> changeMemberName(
      String roomUuid, String userUuid, String name);
}
