// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:json_annotation/json_annotation.dart';
import 'package:netease_roomkit_interface/netease_roomkit_interface.dart';

part 'roomkit_options.g.dart';

///
/// 提供初始化配置
/// [appKey] appKey
/// [useAssetServerConfig] 是否读取私有化配置文件，默认为false
/// [serverConfig] 服务器配置项
/// [serverUrl] 服务器配置项
/// [extras] 预留字段
///
@JsonSerializable(
  createToJson: false,
)
class NERoomKitOptions {
  NERoomKitOptions({
    required this.appKey,
    this.useAssetServerConfig = false,
    this.serverConfig,
    this.serverUrl,
    Map<String, String>? extras,
  }) : extras = extras != null ? Map.from(extras) : const {};

  factory NERoomKitOptions.fromJson(Map map) {
    return _$NERoomKitOptionsFromJson(map.cast());
  }

  /// 应用 appKey
  final String appKey;

  ///
  /// 是否解析 assets 目录下的私有化配置文件
  ///
  final bool useAssetServerConfig;

  ///
  /// 私有化配置
  ///
  final NEServerConfig? serverConfig;

  ///
  /// 私有化配置地址
  ///
  final String? serverUrl;

  /// 额外参数
  final Map<String, String> extras;
}
