// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

// ignore_for_file: unused_element

import 'dart:core';

import 'package:json_annotation/json_annotation.dart';

part 'seat_models.g.dart';

@JsonSerializable()
class NESeatInfo {
  String? creator;
  List<String>? managers;
  List<NESeatItem>? seatItems;

  NESeatInfo(this.creator, this.managers, this.seatItems);

  factory NESeatInfo.fromJson(Map<String, dynamic> json) {
    return _$NESeatInfoFromJson(json);
  }

  @override
  String toString() {
    return 'NESeatInfo{creator: $creator, managers: $managers, seatItems: $seatItems}';
  }

  Map toJson() => _$NESeatInfoToJson(this);
}

@JsonSerializable()
class NESeatItem {
  int? index;
  int? status;
  String? user;
  String? userName;
  String? icon;
  int? onSeatType;
  int? updated;

  NESeatItem(this.index, this.status, this.user, this.userName, this.icon,
      this.onSeatType, this.updated);

  factory NESeatItem.fromJson(Map<String, dynamic> json) {
    return _$NESeatItemFromJson(json);
  }

  @override
  String toString() {
    return 'NESeatItem{index: $index, status: $status, user: $user, userName: $userName, icon: $icon, onSeatType: $onSeatType, updated: $updated}';
  }

  Map toJson() => _$NESeatItemToJson(this);
}

@JsonSerializable()
class NESeatRequestItem {
  int? index;
  String? user;
  String? userName;
  String? icon;

  NESeatRequestItem(this.index, this.user, this.userName, this.icon);

  factory NESeatRequestItem.fromJson(Map<String, dynamic> json) {
    return _$NESeatRequestItemFromJson(json);
  }

  @override
  String toString() {
    return 'NESeatRequestItem{index: $index, user: $user, userName: $userName, icon: $icon}';
  }

  Map toJson() => _$NESeatRequestItemToJson(this);
}

@JsonSerializable()
class NESeatInvitationItem {
  int? index;
  String? user;

  NESeatInvitationItem(this.index, this.user);

  factory NESeatInvitationItem.fromJson(Map<String, dynamic> json) {
    return _$NESeatInvitationItemFromJson(json);
  }

  @override
  String toString() {
    return 'NESeatInvitationItem{index: $index, user: $user}';
  }

  Map toJson() => _$NESeatInvitationItemToJson(this);
}

/// 麦位状态
class NESeatItemStatus {
  /// 麦位初始化（无人，可以上麦）
  static const INITIAL = 0;

  /// 该麦位正在等待管理员通过申请或等待成员接受邀请后上麦。
  static const WAITING = 1;

  /// 当前麦位已被占用
  static const TAKEN = 2;

  /// 当前麦位已关闭，不能操作上麦
  static const CLOSED = -1;
}

/// 上麦类型。
class NESeatOnSeatType {
  /// 无效
  static const INVALID = -1;

  /// 申请上麦
  static const REQUEST = 0;

  /// 邀请上麦
  static const INVITATION = 1;
}

/// 麦位申请审批模式。
/// - [OFF] 审批模式关闭，即成员提交申请后不需要管理员同意，可直接上麦。
/// - [ON]  审批模式打开，即成员提交申请后需要管理员同意才能上麦。
class NESeatRequestApprovalMode {
  static const OFF = 0;
  static const ON = 1;
}

/// 麦位邀请确认模式。
/// - [OFF] 确认模式关闭，即管理员邀请其他成员上麦，不需要被邀请的成员确认，可直接上麦。
/// - [ON]  确认模式打开，即管理员邀请其他成员上麦，需要被邀请的成员确认，接受后方可上麦。
class NESeatInvitationConfirmMode {
  static const OFF = 0;
  static const ON = 1;
}
